@@ -1,4 +1,4 @@
-# COPYRIGHT 2020 by Pietro Mandracci
+# COPYRIGHT (c) 2020-2024 Pietro Mandracci
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -62,7 +62,7 @@ class _PlotWindow:
             plot_type:       a string defining the type of plot
                              must be one of the strings in PLOT_TYPES
             title:           a string used as the title of the plot
-            persistence:     if True, the plot window will not close 
+            persistence:     if True, the window on the screen will not close 
                              after the gnuplot process is terminated
             redirect_output: True:  save gnuplot otuput and errors to files
                              False: send then to /dev/stdout and /dev/stderr
@@ -80,7 +80,7 @@ class _PlotWindow:
             self.persistence:     True if the window is persistent
             self.filename_out:    file to which gnuplot output is redirected 
             self.filename_err:    file to which gnuplot error messages
-                                   are redirected
+                                  are redirected
             self.plot_type:       a string defining the type of plot
             self.n_axes:          number of plot axes (2 for 2D plots, 3 for 3D ones)
             self.xmin:            minimum of x-axis range (initialized to None)
@@ -125,7 +125,7 @@ class _PlotWindow:
         # Store the persistence status
         self.persistence = persistence
 
-        # Store the requested purge status
+        # Store the purge status
         self.purge = purge
         
         # Store the window title (may be None)
@@ -169,6 +169,7 @@ class _PlotWindow:
             # Create the directories to save gnuplot output and errors
             if not path.exists(DIRNAME):     mkdir(DIRNAME)
             if not path.exists(DIRNAME_OUT): mkdir(DIRNAME_OUT)
+            # Create the filenames
             if not path.exists(DIRNAME_ERR): mkdir(DIRNAME_ERR)
             self.filename_out = path.join( DIRNAME_OUT, FILENAME_OUT
                                            + '_w_' + str(self.window_number) )
@@ -177,18 +178,21 @@ class _PlotWindow:
             if self.title is not None:
                 self.filename_out += '(' + correct_filename(self.title) + ')'
                 self.filename_err += '(' + correct_filename(self.title) + ')'
+            # Start the gnuplot process with output redirected to files
             self.gnuplot_process = Popen(exec_list,
                                          stdin=PIPE,
                                          stdout=open(self.filename_out, 'w'),
                                          stderr=open(self.filename_err, 'w'),
                                          universal_newlines=True)
         elif redirect_output is False:
+            # Start the gnuplot process without redirection
             self.filename_out = '/dev/stdout'
             self.filename_err = '/dev/stderr'               
             self.gnuplot_process = Popen(exec_list,
                                          stdin=PIPE,
                                          universal_newlines=True)            
         else:
+            # Start the gnuplot process silenced
             self.filename_out = '/dev/null'
             self.filename_err = '/dev/null'        
             self.gnuplot_process = Popen(exec_list,
@@ -208,14 +212,14 @@ class _PlotWindow:
             
             
     def _command(self, command_string):
-        """ Send a command to the gnuplot process.
-
-            NOTE: no check is made that the string is a valid gnuplot command
+        """ Send a command to the gnuplot process.           
 
             Parameters
             ----------
 
             command_string: string containing a gnuplot command
+                            NOTE: no check is made that the string 
+                            is a valid gnuplot command
         """
 
         self.gnuplot_process.stdin.write(command_string + EOL)
@@ -232,8 +236,8 @@ class _PlotWindow:
         """
 
         try:
-            gnuplot_last_output = self.gnuplot_process.communicate(input='quit',
-                                                                   timeout=TIMEOUT_QUIT)
+            gnuplot_last_output = self.gnuplot_process.communicate(
+                input='quit', timeout=TIMEOUT_QUIT)
         except TimeoutExpired:
             self.gnuplot_process.kill()
             gnuplot_last_output = self.gnuplot_process.communicate()
@@ -300,24 +304,29 @@ class _PlotWindow:
         if replot:                     command_string = 'replot '
         elif (self.plot_type == '3D'): command_string = 'splot '
         else:                          command_string = 'plot '
-              
-        for i in range(len(function_list)):
 
+        command_list = []
+              
+        for function_item in function_list:
+            # Initialize the command string for this fucntion
+            function_command_string = ''
             # Add the function to the function list
-            self.functions.append(str(function_list[i][0]))
+            self.functions.append(str(function_item[0]))
             
             # Read the label, if given
-            if (function_list[i][1] is None): label = None
-            else:                             label = str(function_list[i][1])
+            if (function_item[1] is None): label = None
+            else:                          label = str(function_item[1])
             
             # Add  to the command string the plot command for this function
-            command_string += str(function_list[i][0])
+            function_command_string += str(function_item[0])
             if (label is not None):
-                command_string += ' title \"' + str(label) + '\"'
-            if (function_list[i][2] is not None):
-                command_string += ' ' + str(function_list[i][2])
-            if (i < len(function_list) - 1):
-                command_string += ', '
+                function_command_string += ' title \"' + str(label) + '\"'
+            if (function_item[2] is not None):
+                function_command_string += ' ' + str(function_item[2])
+                
+            command_list.append(function_command_string)
+
+        command_string += ', '.join(command_list)
                         
         # Send command string to gnuplot   
         self._command(command_string)
@@ -338,7 +347,7 @@ class _PlotWindow:
                                      [x2,   y2, z2, label2, options2], ... ] 
                           where:
                           - x1 contains the x-coordinates of the points to plot;
-                            for 2D plot windows only this can be set to None, 
+                            for 2D plot windows this can be set to None, 
                             which is useful if the data are 1D, as in the case 
                             of boxplots, or if you want gnuplot to 
                             automatically provide x-values.
@@ -373,11 +382,18 @@ class _PlotWindow:
         if (replot and (not self.data_filenames) and (not self.functions)):
             return ERROR_NO_REPLOT
         
-        # Check consistency of the whole list
+        # Check consistency of the each data list item
         for i in range(len(data_list)):
+            # Check the number of data given
+            if (len(data_list[i]) != self.n_axes + 2):
+                (status, message) = ERROR_PLOT_PARAMETERS
+                if len(data_list) > 1: message += ' in list item # ' + str(i)
+                message += ( ': ' +  str(self.n_axes + 2)
+                             + ' items expected, given '
+                             + str(len(data_list[i])) )
+                return (status, message)            
             
-            # Check if xdata are missing in this item
-            xmissing = False
+            # Check if xdata are missing in this item            
             if (data_list[i][0] is None):
                 xmissing = True
                 len_x = None
@@ -388,19 +404,20 @@ class _PlotWindow:
                     message += ': missing x-data in a 3D plot window'
                     return status, message
             else:
-                # Transform single x-numbers to lists
+                xmissing = False
+                
+            # Transform single numbers to lists
+            if not xmissing:
                 try:
                     len_x = len(data_list[i][0])
                 except TypeError:
                     data_list[i][0] = [ data_list[i][0] ]
                     len_x = 1
-            # Transform single y-numbers to lists        
             try:
                 len_y = len(data_list[i][1])
             except TypeError:
                 data_list[i][1] = [ data_list[i][1] ]
                 len_y = 1
-            # For a 3D plot, Check z values also
             if (self.n_axes == 3):
                 # Transform single z-numbers to lists
                 try:
@@ -409,15 +426,8 @@ class _PlotWindow:
                     data_list[i][2] = [ data_list[i][2] ]
                     len_z = 1
                     
-            # Check consistency of the data        
-            if (len(data_list[i]) != self.n_axes + 2):
-                (status, message) = ERROR_PLOT_PARAMETERS
-                if len(data_list) > 1: message += ' in list item # ' + str(i)
-                message += ( ': ' +  str(self.n_axes + 2)
-                             + ' items expected, given '
-                             + str(len(data_list[i])) )
-                return (status, message)
-            elif ( (not xmissing) and (len_x != len_y) ):
+            # Check consistency of the data lengths
+            if ( (not xmissing) and (len_x != len_y) ):
                 (status, message) = ERROR_PLOT_PARAMETERS
                 if len(data_list) > 1:  message += ' in list item # ' + str(i)
                 message += ': x and y data have different sizes'
@@ -459,42 +469,48 @@ class _PlotWindow:
         elif (self.plot_type == '3D'): command_string = 'splot '
         else:                          command_string = 'plot '
 
+        # Initialize the list in which commands for each curve
+        # will be stored
+        command_list = []
+        
         # Save the old number of curves before adding the new ones
-        n_curves = len(self.data_filenames)          
-        for i in range(len(data_list)):         
-                        
+        n_curves = len(self.data_filenames)       
+        for data_item in data_list:
+            # Initialize the string for this curve
+            curve_command_string = ''
+            
             # Read the curve label, if given
-            if (data_list[i][self.n_axes] is None):
+            if (data_item[self.n_axes] is None):
                 label = None
             else:
-                label = str(data_list[i][self.n_axes])
+                label = str(data_item[self.n_axes])
 
             # Read additional options, if given
-            if (data_list[i][self.n_axes+1] is None):
+            if (data_item[self.n_axes+1] is None):
                 options = None
             else:
-                options = str(data_list[i][self.n_axes+1])
+                options = str(data_item[self.n_axes+1])
                         
             # Define the unique filename for the data file
-            # or "-" for volatile data
+            # or use the gnuplot special filename '-' for volatile data
             if volatile:
                 filename = FILENAME_VOLATILE
             else:
                 curve_number = n_curves + i
-                string = FILENAME_DATA 
-                string += '_w' + str(self.window_number)
-                if (self.plot_type == '3D')   : string += '_3D'
-                elif (data_list[i][0] is None): string += '_1D'
-                else:                           string += '_2D'        
+                name_string = FILENAME_DATA 
+                name_string += '_w' + str(self.window_number)
+                if (self.plot_type == '3D'): name_string += '_3D'
+                elif (data_item[0] is None): name_string += '_1D'
+                else:                        name_string += '_2D'        
                 if self.title is not None:
-                    string += '(' + correct_filename(self.title) + ')'
-                string += '_c' + str(curve_number)
+                    name_string += '(' + correct_filename(self.title) + ')'
+                name_string += '_c' + str(curve_number)
                 if label is not None:
-                    string += '(' + correct_filename(label) + ')' 
-                string += FILENAME_DATA_EXT
-                filename = path.join( DIRNAME_DATA, string )
+                    name_string += '(' + correct_filename(label) + ')' 
+                name_string += FILENAME_DATA_EXT
+                filename = path.join( DIRNAME_DATA, name_string )
 
-            # Add filename to the list, or increase volatile data counter
+            # Add filename to list, or increase volatile data counter
             if volatile:
                 self.n_volatiles += 1
             else:
@@ -503,32 +519,34 @@ class _PlotWindow:
             # Save data to data files, unless they are volatile
             if not volatile:
                 if (self.plot_type == '3D'):
-                    data_file_3d( data_list[i][0],
-                                  data_list[i][1],
-                                  data_list[i][2],
+                    data_file_3d( data_item[0],
+                                  data_item[1],
+                                  data_item[2],
                                   filename )
 
                 elif (data_list[i][0] is None):
-                    data_file_1d( data_list[i][1],
+                    data_file_1d( data_item[1],
                                   filename )
 
                 else:
-                    data_file_2d( data_list[i][0],
-                                  data_list[i][1],
+                    data_file_2d( data_item[0],
+                                  data_item[1],
                                   filename )
 
-            # Add plot command to the command string
-            command_string += '\"' + filename + '\"'
+            # Add plot command to the command list
+            curve_command_string += '\"' + filename + '\"'
             if (label is None):
                 # If we don't set a title, gnuplot automatically
                 # uses the data file name, so we set "" as title
-                command_string += ' title \"\"'
+                curve_command_string += ' title \"\"'
             else:
-                command_string += ' title \"' + label + '\"'
+                curve_command_string += ' title \"' + label + '\"'
             if (options is not None):
-                command_string += ' ' + options
-            if (i < len(data_list) - 1):
-                command_string += ', '
+                curve_command_string += ' ' + options
+                
+            command_list.append(curve_command_string)
+
+        command_string += ', '.join(command_list)
 
         # For volatile data, add the datastrings          
         if volatile:
@@ -536,15 +554,15 @@ class _PlotWindow:
             for data in data_list:
                 if (self.plot_type == '3D'):
                     command_string += data2string_3d(data[0],
-                                                     data[1],
-                                                     data[2])
+                                                   data[1],
+                                                   data[2])
 
                 elif (data[0] is None):
                     command_string += data2string_1d(data[1])
 
                 else:
                     command_string += data2string_2d(data[0],
-                                                     data[1])
+                                                   data[1])
                         
         # Send command string to gnuplot   
         self._command(command_string)
